% Script to assemble the gridwave file for use in usfos. Calculated input
% in use is elevation in time/space, gridwave setup variables, velocity and
% acceleration amplitudes and phase angles, time input, and calculation
% input.
% Velocity and acceleration profiles are calculated in script, for each
% time step, but wave elevation is precalculated for each time step.

% first part of the script defines grid size, mesh and which data for the
% gridwave file to be considered. Then wave elevation, velocity profile and acceleration profile 
% is calculated in space, for each time step.

% An explanation of the set up of the file
% is given in USFOS documentation, "wave kinematics file format.pdf" as a
% separate pdf file along with the script files.


function gridwrite(elev,xmin,xmax,dx,ny,nx,dt,ymin,ymax,u,Phase1,kn,d,samp,N,w1)
maxpeak=max(max(elev)); % determine max and min wave elevation. heighest and lowest elevation determines were to start increased resolution in vertical nodes of the grid
minpeak=min(min(elev));
time=0; % time starts at 0, yet the calculated time series is calculated from t=500.
z=[-150,-125,-100,-75,-50,-45,-40,-35,-34:2:-20,(minpeak-3):0.5:(maxpeak+0.5)]; % depth profile, increased resolution near and around surface/elevation
fid=fopen(['gwf',num2str(N),'_',num2str(samp),'.w132'],'w+'); % open/create writeable file to store gridwave input, filname decided by amount of components and samplenumber

%starting to write the input file:
fprintf(fid,'''\t\t GWF Type \n');
fprintf(fid,' GWF\tType\t132\n');
fprintf(fid,'''\n');
fprintf(fid,'''\t\t\t Type\tnX\tnY\tnZ\tnStep\tParameter code\t(Elev,Velo,Acc,Press)\n');
fprintf(fid,[' GWF   KeyData    Size  ',num2str(nx),'   ',num2str(ny),'  ',num2str(length(z)),'  ',num2str(size(elev,1)),'  1101 \n']); % defining the mesh for the gridwave
fprintf(fid,'''\n');
fprintf(fid,'''\n');
fprintf(fid,[' GWF   Grid   X   Constant  ',num2str(xmin),'  ',num2str(xmax),'\n']); % assigning the metric area for the wave kinematic mesh
fprintf(fid,[' GWF   Grid   Y   Constant  ',num2str(ymin),'  ',num2str(ymax),'\n']); % in x, y  and z direction
fprintf(fid,' GWF   Grid   Z   Free \n'); % nodes are assigned freely in vertical direction, less spacing near surface
for i=1:(floor(length(z)/12)); % writing the node values for z
    fprintf(fid,' %.1f ', z(1+(i-1)*12:12+(i-1)*12));
    fprintf(fid,'\n');
end
fprintf(fid,' %.1f', z((i*12)+1:end));
fprintf(fid,'\n');
% for each time step, data for elevation, velocity and acceleration are written. the setup of the data group is defined in the above referenced pdf 
fprintf(fid,'''\n');
fprintf(fid,'''\n');
for i=1:size(elev,1);
    time=(i-1)*dt;
    fprintf(fid,' GWF    Time    %.2f \n', time);
    fprintf(fid,'\n');
    fprintf(fid,' GWF    Elevation \n');
    for lll=1:nx;
        fprintf(fid,[' ',num2str(elev(i,lll)),' \n']);
    end  
    
    fprintf(fid,'''\n');
    fprintf(fid,' GWF    Velocity \n');
    fprintf(fid,'''\n');
   for k=1:nx; % adjusting depth in accordance with surface elevation(airy stretching)
    stretch(:,k)=(z-elev(i,k))./(d+elev(i,k))*d;
   end
    % calc velocity depth profile for all x nodes, only horizontal velocity
    for jj=1:nx;
        for jjj=1:length(stretch);
            if stretch(jjj,jj)<=0
                ux(jjj,1)=sum(u.*cosh(kn.*(stretch(jjj,jj)+d))./cosh(kn.*d).*cos(-w1.*time+(Phase1*pi/180)));
            else
                ux(jjj,1)=0;
            end
            ux(jjj,2)=0;
            ux(jjj,3)=0;
            fprintf(fid,' %f ',ux(jjj,:));
            fprintf(fid,'\n');
        end
        fprintf(fid,'\n');
    end
    %write acceleration nodes:
    fprintf(fid,'''\n');
    fprintf(fid,' GWF    Acceleration \n');
    for jj=1:nx;
        for jjj=1:length(stretch);
            if stretch(jjj,jj)<=0
                ax(jjj,1)=sum(w1.*u.*cosh(kn.*(stretch(jjj,jj)+d))./cosh(kn.*d).*sin(-w1.*time+(Phase1*pi/180)));
            else
                ax(jjj,1)=0;
            end
            ax(jjj,2)=0;
            ax(jjj,3)=0;
            fprintf(fid,' %f ',ax(jjj,:));
            fprintf(fid,'\n');
        end
        fprintf(fid,'\n');
    end
    fprintf(fid,'''\n');
    fprintf(fid,'''\n');

    end

fprintf(fid,''' \n');
fprintf(fid,' GWF    End \n');
fprintf(fid,''' \n');
fprintf(fid,''' \n');
fprintf(fid,''' -------------- EOF --------------');
fclose(fid);
movefile(['gwf',num2str(N),'_',num2str(samp),'.w132'],'gridwave/inp/dx10');
end   


    
    


