% Script to generate wave components with random amplitudes
% and random frequenies. Spectrum is JONSWAP. Components are assigned
% random phase angles from 0 to 2*pi. Components are written to files for
% use in USFOS calculations. Amount of components in each data set are
% varied, and 20 unique datasets are generated for the different amount of
% components used. The method for calcuating linear wave components are
% based on recommended practice from DNV, DNV-RP-C205.


% Create Jonswap spectrum with parameters
%Tp = 17s
% Hs= 16.4
% gamma = 3.3
% Frequency range from 0.125/s to 2.7/s


% other dependent parameters
% sigma, sigmaA, sigmaB, V, g, alpha
%alpha dependent on Tp and Hm0, which is known values. linear variations
%over Tp is assumed(Myrhaug 2007). The spectrum is based on the code given
%in the paper; USFOS hydrodynamics eq. (1.48)
clear all;
if ~exist('randampfreq')
  mkdir('randampfreq'); % create folder to store txt-files with wavecomponent input for usfos

end

sigmaA=0.07;
sigmaB=0.09;
sigma=0.08;
g=9.81;
Tp=17;
Hs=16.4;
gamma=3.3;
delta=0.036-(0.0056*(Tp/sqrt(Hs))); % factor for JONSWAP
gamma1=exp(3.483*(1-(0.1975*delta*(Tp^4)/(Hs^2)))); % factor for JONSWAP
alpha=5.061*(1-(0.287*log(gamma)))*(Hs^2)/(Tp^4); % factor for JONSWAP spectrum
omegap=2*pi/Tp; % peak frequency
omegalo=2*pi/50; %lowest wave frequency in the spectrum 
omegaup=2*pi/3; % highgest wave frequency in th spectrum
N1=75; 
N2=100; %N1-N3 are number of wave components to be calculated
N3=200;
N4=1000;
samples=20; % amount of samples to generate for each set of components

% Calculating a matrix of spectral periods used to calculate the wave
% component amplitudes. The frequency interval for a given component is
% constant, and set depending on amount of components calculated(75, 100,
% 200). The wave frequency is selcted to be a random value within the given
% frequency interval used to calculate the amplitude(ref. Shao/Faltinsen).
% The individual wave components are also assigned a random phase angle,
% and will increase the randomness of the components.

delta_w1=(omegaup-omegalo)/N1;
delta_w2=(omegaup-omegalo)/N2; % frequency interval width corresponding to N# wave components/step length
delta_w3=(omegaup-omegalo)/N3;
delta_w4=(omegaup-omegalo)/N4;


Dir=0; % variable assigning direction of propagation. set to zero for all components/datasets in this project.

% Calculating the wave ampltidues for N1 = 75 components. Calculation based on method
% suggested in zhao's paper, and with random amplitude method suggested by DNV. The frequency, w1, to calculate the spectrum
% is selected randomly within the frequencyinterval dw for each component.
% and the random amplitude follows a rayleigh distribution with mean-square
% value E[(amp_k)^2]=2*S(w_k)*dw_k
for k=1:20;
    fname=sprintf('Spec75_%d.txt', k);
    FileID=fopen(fname, 'w');
    
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
        for i=1:N1;
        w1(i,1)=omegalo+((i-1)*delta_w1)+(delta_w1*rand); %amplitude selected at random within frequency interval of each component
            if w1(i,1)<=omegap;
        factor1(i,1)=exp((-1)*(((w1(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
           else
        factor1(i,1)=exp((-1)*(((w1(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
            end
        gammaf1(i,1)=gamma^factor1(i,1);
        S_N1(i,k)=alpha*g*g*(w1(i,1)^(-5))*exp((-1.25)*((w1(i,1)/omegap)^(-4)))*gammaf1(i,1);
        Wavamp_N1(i,k)=sqrt(2*S_N1(i,k)*delta_w1); %temporary amp
        randAmp_N1(i,k)=raylinv(rand,sqrt((Wavamp_N1(i,k)^2)/2)); %randomization of amplitude based on rayleigh distribution
        Phase1(i,k)=rand*360;
        Period1(i,k)=2*pi/w1(i,1);
    
        fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, randAmp_N1(i,k), Period1(i,k), Dir, Phase1(i,k));
        end
        fprintf(FileID, '\n#');
    fclose(FileID);        
    movefile(fname,'randampfreq');
end

% For N2 = 100 components, for 20 samples:
for k=1:20;
    fname=sprintf('Spec100_%d.txt', k);
    FileID=fopen(fname,'w');
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
    for i=1:N2;
        w2(i,1)=omegalo+((i-1)*delta_w2)+(delta_w2*rand);
        if w2(i,1)<=omegap;
            factor2(i,1)=exp((-1)*(((w2(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
        else
            factor2(i,1)=exp((-1)*(((w2(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
        end
        gammaf2(i,1)=gamma^factor2(i,1);
        S_N2(i,k)=alpha*g*g*(w2(i,1)^(-5))*exp((-1.25)*((w2(i,1)/omegap)^(-4)))*gammaf2(i,1);
        Wavamp_N2(i,k)=sqrt(2*S_N2(i,k)*delta_w2); %temporary amp
        randAmp_N2(i,k)=raylinv(rand,sqrt((Wavamp_N2(i,k)^2)/2)); %randomization of amplitude based on rayleigh distribution
        Phase2(i,k)=rand*360; 
        Period2(i,k)=2*pi/w2(i,1);
        fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, randAmp_N2(i,k), Period2(i,k), Dir, Phase2(i,k));

    end
    fprintf(FileID, '\n#');
    fclose(FileID);
    movefile(fname,'randampfreq');
end

% 200 Components

for k=1:20;
    fname=sprintf('Spec200_%d.txt', k);
    FileID=fopen(fname,'w');
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
    for i=1:N3;
    w3(i,1)=omegalo+((i-1)*delta_w3)+(delta_w3*rand);
    if w3(i,1)<=omegap;
        factor3(i,1)=exp((-1)*(((w3(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
    else
        factor3(i,1)=exp((-1)*(((w3(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
    end
    gammaf3(i,1)=gamma^factor3(i,1);
    S_N3(i,k)=alpha*g*g*(w3(i,1)^(-5))*exp((-1.25)*((w3(i,1)/omegap)^(-4)))*gammaf3(i,1);
    Wavamp_N3(i,k)=sqrt(2*S_N3(i,k)*delta_w3);
    randAmp_N3(i,k)=raylinv(rand,sqrt((Wavamp_N3(i,k)^2)/2));
    Phase3(i,k)=rand*360;
    Period3(i,k)=2*pi/w3(i,1);
    fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, randAmp_N3(i,k), Period3(i,k), Dir, Phase3(i,k));

    end
    fprintf(FileID, '\n#');
    fclose(FileID);
    movefile(fname,'randampfreq');
end

% 1000 comp
    
for k=1:20;
    fname=sprintf('Spec1000_%d.txt', k);
    FileID=fopen(fname,'w');
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
    for i=1:N4;
    w4(i,1)=omegalo+((i-1)*delta_w4)+(delta_w4*rand);
    if w4(i,1)<=omegap;
        factor4(i,1)=exp((-1)*(((w4(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
    else
        factor4(i,1)=exp((-1)*(((w4(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
    end
    gammaf4(i,1)=gamma^factor4(i,1);
    S_N4(i,k)=alpha*g*g*(w4(i,1)^(-5))*exp((-1.25)*((w4(i,1)/omegap)^(-4)))*gammaf4(i,1);
    Wavamp_N4(i,k)=sqrt(2*S_N4(i,k)*delta_w4);
    randAmp_N4(i,k)=raylinv(rand,sqrt((Wavamp_N4(i,k)^2)/2));
    Phase4(i,k)=rand*360;
    Period4(i,k)=2*pi/w4(i,1);
    fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, randAmp_N4(i,k), Period4(i,k), Dir, Phase4(i,k));

    end
    fprintf(FileID, '\n#');
    fclose(FileID);
    movefile(fname,'randampfreq');
end